(() => {
    if (typeof browser == "undefined") {
        var browser = chrome;
    }
    /**
     * Content Script used on MetaGer settings page
     */
    document.querySelectorAll("form").forEach(form => {
        form.addEventListener("submit", e => {
            e.preventDefault();
            submitForm(form);
        });
    });

    // Remove change listeners on all select elements
    // And add our own
    document.querySelectorAll("select").forEach(select => {
        let element = select.cloneNode(true);
        select.parentNode.replaceChild(element, select);
        element.addEventListener("change", e => {
            submitForm(element.form);
        })
    });

    // Make the remove key button work
    document.querySelectorAll("a").forEach(anchor => {
        anchor.addEventListener("click", e => {
            try {
                let url = new URL(anchor.href);
                if (url.pathname == "/keys/key/remove") {
                    e.preventDefault(); // Prevent direct loading of page
                    browser.runtime.sendMessage({ type: "settings_remove", setting_key: "key" }).then(() => {
                        document.location.href = url;
                    })
                }
            } catch (error) { }
        })
    });

    function submitForm(form) {
        let form_action = form.action;

        const form_data = new URLSearchParams();
        for (const pair of new FormData(form)) {
            form_data.append(pair[0], pair[1]);
        }

        fetch(form_action, {
            method: "POST",
            headers: {
                "Accept": "application/json"
            },
            credentials: "include",
            redirect: "follow",
            body: form_data
        })
            .then(response => response.json())
            .then(async response => {
                for (let remove of response.remove) {
                    await browser.runtime.sendMessage({ type: "settings_remove", setting_key: remove });
                }
                if (Object.keys(response.set).length > 0) {
                    return browser.runtime.sendMessage({ type: "settings_set", settings: response.set })
                        .then(answer => {
                            return response.redirect
                        });
                } else {
                    return response.redirect;
                }
            }).then((url) => {
                if (url) {
                    if (!url.match(/#.*/) && form.id) {
                        url += "#form.id";
                    }
                    document.location.href = url;
                }
                document.location.reload();
            })
            .catch(error => {
                //form.submit();
                console.log(error);
            })
    }
})();